@@ -96,13 +96,13 @@ pub unsafe fn write_float_scientific<F: DragonboxFloat, const FORMAT: u128>(
     // Write the significant digits. Write at index 1, so we can shift 1
     // for the decimal point without intermediate buffers.
     // SAFETY: safe, if we have enough bytes to write the significant digits.
-    let mut digits = unsafe { &mut index_unchecked_mut!(bytes[1..]) };
+    let digits = unsafe { &mut index_unchecked_mut!(bytes[1..]) };
     let digit_count = unsafe { F::write_digits(digits, fp.mant) };
 
     // Truncate and round the significant digits.
     // SAFETY: safe since `digit_count < digits.len()`.
     let (digit_count, carried) =
-        unsafe { shared::truncate_and_round_decimal(&mut digits, digit_count, options) };
+        unsafe { shared::truncate_and_round_decimal(digits, digit_count, options) };
     let sci_exp = sci_exp + carried as i32;
 
     // Determine the exact number of digits to write.
@@ -343,47 +343,61 @@ pub fn to_decimal<F: RawFloat>(float: F) -> ExtendedFloat80 {
         return extended_float(0, 0);
     }
 
-    // Round-nearest, tie-even case:
-    //
-    // Shorter interval case; proceed like Schubfach. One might think this
-    // condition is wrong, since when exponent_bits == 1 and two_fc == 0,
-    // the interval is actually regular. However, it turns out that this
-    // seemingly wrong condition is actually fine, because the end result is
-    // anyway the same.
+    // Shorter interval case; proceed like Schubfach.
+    // One might think this condition is wrong, since when exponent_bits == 1
+    // and two_fc == 0, the interval is actullay regular. However, it turns out
+    // that this seemingly wrong condition is actually fine, because the end
+    // result is anyway the same.
     //
     // [binary32]
-    // floor( (fc-1/2) * 2^e ) = 1.175'494'28... * 10^-38
-    // floor( (fc-1/4) * 2^e ) = 1.175'494'31... * 10^-38
-    // floor(    fc    * 2^e ) = 1.175'494'35... * 10^-38
-    // floor( (fc+1/2) * 2^e ) = 1.175'494'42... * 10^-38
+    // (fc-1/2) * 2^e = 1.175'494'28... * 10^-38
+    // (fc-1/4) * 2^e = 1.175'494'31... * 10^-38
+    //    fc    * 2^e = 1.175'494'35... * 10^-38
+    // (fc+1/2) * 2^e = 1.175'494'42... * 10^-38
     //
     // Hence, shorter_interval_case will return 1.175'494'4 * 10^-38.
-    // 1.175'494'3 * 10^-38 is also a correct shortest representation that
-    // will be rejected if we assume shorter interval, but 1.175'494'4 *
-    // 10^-38 is closer to the true value so it doesn't matter.
+    // 1.175'494'3 * 10^-38 is also a correct shortest representation that will
+    // be rejected if we assume shorter interval, but 1.175'494'4 * 10^-38 is
+    // closer to the true value so it doesn't matter.
     //
     // [binary64]
-    // floor( (fc-1/2) * 2^e ) = 2.225'073'858'507'201'13... * 10^-308
-    // floor( (fc-1/4) * 2^e ) = 2.225'073'858'507'201'25... * 10^-308
-    // floor(    fc    * 2^e ) = 2.225'073'858'507'201'38... * 10^-308
-    // floor( (fc+1/2) * 2^e ) = 2.225'073'858'507'201'63... * 10^-308
+    // (fc-1/2) * 2^e = 2.225'073'858'507'201'13... * 10^-308
+    // (fc-1/4) * 2^e = 2.225'073'858'507'201'25... * 10^-308
+    //    fc    * 2^e = 2.225'073'858'507'201'38... * 10^-308
+    // (fc+1/2) * 2^e = 2.225'073'858'507'201'63... * 10^-308
     //
-    // Hence, shorter_interval_case will return 2.225'073'858'507'201'4 * 10^-308.
-    // This is indeed of the shortest length, and it is the unique one
-    // closest to the true value among valid representations of the same
-    // length.
-
-    // NOTE: unlike Dragonbox, we don't need to check exponent_bits,
-    // since if both the exponent bits and mantissa bits are 0, we have
-    // a literal 0.
+    // Hence, shorter_interval_case will return 2.225'073'858'507'201'4 *
+    // 10^-308. This is indeed of the shortest length, and it is the unique one
+    // closest to the true value among valid representations of the same length.
 
     // Toward zero case:
     //
-    // According to the main Dragonbox algorithm, this is the left-closed
-    // directed rounding case, however, that is the case of I = [w,w+), which
-    // does not round-down. Rather, we want the right-closed directed rounding
-    // case, or I = (w−,w]. Unfortunately, this doesn't work, since it truncates
-    // **too** much, so we just round-nearest, tie-even, then direct the rounding.
+    // What we need is a compute-nearest, but with truncated digits in the
+    // truncated case. Note that we don't need the left-closed direct
+    // rounding case of I = [w,w+), or right-closed directed rounding
+    // case of I = (w−,w], since these produce the shortest intervals for
+    // a **float parser** assuming the rounding of the float-parser.
+    // The left-directed case assumes the float parser will round-down,
+    // while the right-directed case assumed the float parser will round-up.
+    //
+    // A few examples of this behavior is described here:
+    //    **compute_nearest_normal**
+    //
+    //    - `1.23456 => (123456, -5)` for binary32.
+    //    - `1.23456 => (123456, -5)` for binary64.
+    //    - `13.9999999999999982236431606 => (13999999999999998, -15)` for binary64.
+    //
+    //     **compute_left_closed_directed**
+    //
+    //    - `1.23456 => (12345601, -7)` for binary32.
+    //    - `1.23456 => (12345600000000002, -16)` for binary64.
+    //    - `13.9999999999999982236431606 => (13999999999999999, -15)` for binary64.
+    //
+    //     **compute_right_closed_directed**
+    //
+    //    - `1.23456 => (123456, -5)` for binary32.
+    //    - `1.23456 => (123456, -5)` for binary64.
+    //    - `13.9999999999999982236431606 => (13999999999999982, -15)` for binary64.
 
     if mantissa_bits.as_u64() == 0 {
         compute_round_short(float)
@@ -680,9 +694,13 @@ pub fn compute_right_closed_directed<F: RawFloat>(float: F, shorter: bool) -> Ex
         return short_circuit();
     } else if r == deltai {
         // Compare the fractional parts.
-        if shorter && F::compute_mul_parity(two_fc * 2 - 1, &pow5, beta_minus_1 - 1) {
-            return short_circuit();
-        } else if !shorter && F::compute_mul_parity(two_fc - 1, &pow5, beta_minus_1) {
+        let two_f = two_fc
+            - if shorter {
+                1
+            } else {
+                2
+            };
+        if F::compute_mul_parity(two_f, &pow5, beta_minus_1) {
             return short_circuit();
         }
     }
@@ -812,7 +830,7 @@ pub const fn umul192_middle64(x: u64, hi: u64, lo: u64) -> u64 {
 
 #[inline(always)]
 pub const fn umul96_upper32(x: u64, y: u64) -> u64 {
-    umul128_upper64(x, y)
+    umul128_upper64(x, y) as u32 as _
 }
 
 #[inline(always)]
@@ -823,9 +841,6 @@ pub const fn umul96_lower64(x: u64, y: u64) -> u64 {
 // LOG
 // ---
 
-// These are much more efficient log routines than the ones
-// provided by dragonbox, since they use only a mul and shr.
-
 /// Calculate `x * log5(2)` quickly.
 /// Generated by `etc/log.py`.
 /// Only needs to be valid for values from `[-1492, 1492]`
@@ -858,13 +873,13 @@ pub const fn floor_log5_pow2_minus_log5_3(q: i32) -> i32 {
     q.wrapping_mul(451597).wrapping_sub(715764) >> 20
 }
 
-/// Calculate `(x * log10(2) - log10(4)) / 3` quickly.
+/// Calculate `(x * log10(2) - log10(4 / 3))` quickly.
 /// Generated by `etc/log.py`.
 /// Only needs to be valid for values from `[-1700, 1700]`
 #[inline(always)]
 pub const fn floor_log10_pow2_minus_log10_4_over_3(q: i32) -> i32 {
     // NOTE: these values aren't actually exact:
-    //      They're off sfor -295 and 97, so any automated way of computing
+    //      They're off for -295 and 97, so any automated way of computing
     //      them will also be off.
     q.wrapping_mul(1262611).wrapping_sub(524031) >> 22
 }
@@ -1108,6 +1123,11 @@ pub const fn low(pow5: &(u64, u64)) -> u64 {
 /// Calculate the maximum possible power for the mantissa.
 #[inline(always)]
 pub fn max_power<F: DragonboxFloat>() -> i32 {
+    //  NOTE:
+    //      Dragonbox's reference implementation uses
+    //      `max_mantissa = max / pow64(10, F::KAPPA + 1)`, but then does
+    //      `p < max_mantissa / 10`, which produces the same result in all
+    //      cases.
     let max = F::Unsigned::MAX.as_u64();
     let max_mantissa = max / pow64(10, F::KAPPA + 2);
     let mut k = 0i32;
